var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source)
    if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0)
      target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(source)) {
      if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop))
        target[prop] = source[prop];
    }
  return target;
};

// src/type-guards.ts
var isOptionElement = (element) => element.tagName === "OPTION";
var isInputElement = (element) => element.tagName === "INPUT";
var isDetailsElement = (element) => element.tagName.toLowerCase() === "details";
var isSummaryElement = (element) => element.tagName.toLowerCase() === "summary";
var isDefined = (value) => value !== void 0 && value !== null;
var isTextMatcher = (value) => typeof value === "string" || typeof value === "number" || value instanceof RegExp || typeof value === "function";

// src/helpers/by-role.ts
function anyObjectPropertiesAreDefined(obj) {
  return Object.values(obj).some((value) => value !== void 0);
}
function byRole(role, nameOrPropertiesOrChildren, childrenIfProvided) {
  const a11yNode = {
    role
  };
  if (typeof nameOrPropertiesOrChildren === "undefined") {
    return a11yNode;
  } else if (isTextMatcher(nameOrPropertiesOrChildren)) {
    a11yNode.name = nameOrPropertiesOrChildren;
    if (Array.isArray(childrenIfProvided)) {
      a11yNode.children = childrenIfProvided;
    }
  } else if (Array.isArray(nameOrPropertiesOrChildren)) {
    a11yNode.children = nameOrPropertiesOrChildren;
  } else if (typeof nameOrPropertiesOrChildren === "object") {
    const _a = nameOrPropertiesOrChildren, { name, description } = _a, stateAndQueries = __objRest(_a, ["name", "description"]);
    a11yNode.name = name;
    a11yNode.description = description;
    const _b = stateAndQueries, { level, value } = _b, state = __objRest(_b, ["level", "value"]);
    const queries = { level, value };
    a11yNode.state = anyObjectPropertiesAreDefined(state) ? state : void 0;
    a11yNode.queries = anyObjectPropertiesAreDefined(queries) ? queries : void 0;
    if (Array.isArray(childrenIfProvided)) {
      a11yNode.children = childrenIfProvided;
    }
  }
  return a11yNode;
}

// src/tree/leafs.ts
var StaticText = class {
  constructor(text) {
    this.text = text;
  }
  toString() {
    if (this.text === null) {
      return "null";
    }
    return this.text;
  }
};

// src/helpers.ts
import _deepClone from "lodash.clonedeep";
import _isEqual from "lodash.isequal";
var containerAttributeValues = {
  name: "",
  role: "generic",
  description: ""
};
var defaultState = {
  busy: false,
  checked: void 0,
  current: false,
  disabled: false,
  expanded: void 0,
  pressed: void 0,
  selected: void 0
};
var defaultQueries = {
  level: void 0,
  value: {
    min: void 0,
    max: void 0,
    now: void 0,
    text: void 0
  }
};
var isDefaultState = (state) => {
  return _isEqual(state, defaultState);
};
var isDefaultQueries = (queries) => {
  return _isEqual(queries, defaultQueries);
};
var isDefaultAttributeValues = (node) => {
  return (node.role === containerAttributeValues.role || node.role === void 0) && node.name === containerAttributeValues.name && node.description === containerAttributeValues.description;
};
var isContainer = (node) => {
  return isDefaultAttributeValues(node) && isDefaultState(node.state) && isDefaultQueries(node.queries);
};

// src/pretty-tree/omit-default-values.ts
function isNestedObject(value) {
  return typeof value === "object" && value !== null && !(value instanceof RegExp);
}
function omitDefaultValues(target, defaults) {
  const filteredObject = {};
  Object.keys(target).forEach((key) => {
    const targetValue = target[key];
    const defaultValue = defaults[key];
    if (targetValue !== defaultValue && targetValue !== void 0) {
      filteredObject[key] = targetValue;
    } else if (isNestedObject(targetValue) && isNestedObject(defaultValue)) {
      const result = omitDefaultValues(targetValue, defaultValue);
      if (Object.keys(result).length > 0) {
        filteredObject[key] = result;
      }
    }
  });
  return filteredObject;
}

// src/pretty-tree/render-properties.ts
var renderPropertyValue = (value) => {
  switch (typeof value) {
    case "function":
      return "Function";
    case "number":
    case "boolean":
      return String(value);
    case "undefined":
      return "";
    case "object":
      if (value instanceof RegExp) {
        return value.toString();
      }
      return "";
    default:
      return `"${value}"`;
  }
};
var renderProperties = (obj, prefix = "") => {
  const keyValuePairs = Object.entries(obj).flatMap(([key, value]) => {
    if (value === void 0) {
      return [];
    }
    const fullKey = prefix ? `${prefix}.${key}` : key;
    if (typeof value === "object" && !(value instanceof RegExp) && !(value instanceof Function)) {
      return renderProperties(value, fullKey);
    } else {
      const renderedValue = renderPropertyValue(value);
      return renderedValue ? `${fullKey}=${renderedValue}` : [];
    }
  });
  return keyValuePairs.join(" ");
};

// src/pretty-tree/pretty-tree.ts
var renderStringMatcher = (textMatcher) => {
  if (textMatcher instanceof RegExp || typeof textMatcher === "number") {
    return textMatcher.toString();
  }
  if (typeof textMatcher === "function") {
    return textMatcher.toString();
  }
  return `"${textMatcher}"`;
};
var getPrettyTree = (tree, depth = 0) => {
  const indentation = "  ".repeat(depth);
  if (typeof tree === "string" || typeof tree === "number") {
    return `${indentation}"${tree}"
`;
  }
  if (tree instanceof RegExp) {
    return `${indentation}${tree.toString()}
`;
  }
  if (tree instanceof StaticText) {
    return `${indentation}StaticText "${tree.text}"
`;
  }
  if (tree === void 0) {
    return `${indentation}undefined
`;
  }
  if (typeof tree === "function") {
    return `${indentation}${tree.toString()}
`;
  }
  const filteredState = tree.state ? omitDefaultValues(tree.state, defaultState) : void 0;
  const filteredQueries = tree.queries ? omitDefaultValues(tree.queries, defaultQueries) : void 0;
  const nameString = tree.name ? ` ${renderStringMatcher(tree.name)}` : "";
  const stateString = filteredState ? ` ${renderProperties(filteredState)}` : "";
  const queriesString = filteredQueries ? ` ${renderProperties(filteredQueries)}` : "";
  let output = `${indentation}${tree.role}${nameString}${stateString}${queriesString}
`;
  if (tree.description) {
    output += `${indentation}  description: ${renderStringMatcher(
      tree.description
    )}
`;
  }
  if (tree.children) {
    tree.children.forEach((child) => {
      output += getPrettyTree(child, depth + 1);
    });
  }
  return output;
};

// src/tree/role-helpers.ts
import { elementRoles } from "aria-query";

// src/tree/virtual-roles.ts
var nonLandmarkVirtualRoles = [
  // https://w3c.github.io/html-aria/#el-header
  // header: 'HeaderAsNonLandmark',
  {
    match: (node) => {
      return node.tagName.toLowerCase() === "header";
    },
    roles: ["HeaderAsNonLandmark"],
    specificity: 1
  },
  // https://w3c.github.io/html-aria/#el-footer
  // footer: 'FooterAsNonLandmark',
  {
    match: (node) => node.tagName.toLowerCase() === "footer",
    roles: ["FooterAsNonLandmark"],
    specificity: 1
  }
];
var nonListListItemRoles = [
  // li: 'generic',
  // https://www.w3.org/TR/wai-aria-1.2/#listitem
  {
    match: (node) => node.tagName.toLowerCase() === "li",
    roles: ["generic"],
    specificity: 1
  }
];
var virtualRoles = [
  // abbr: 'Abbr'
  {
    match: (node) => node.tagName.toLowerCase() === "abbr",
    roles: ["Abbr"],
    specificity: 1
  },
  // audio: 'Audio',
  {
    match: (node) => node.tagName.toLowerCase() === "audio",
    roles: ["Audio"],
    specificity: 1
  },
  // canvas: 'Canvas',
  {
    match: (node) => node.tagName.toLowerCase() === "canvas",
    roles: ["Canvas"],
    specificity: 1
  },
  // details: 'Details',
  {
    match: (node) => node.tagName.toLowerCase() === "details",
    roles: ["Details"],
    specificity: 1
  },
  // dd: 'DescriptionListDetails',
  {
    match: (node) => node.tagName.toLowerCase() === "dd",
    roles: ["DescriptionListDetails"],
    specificity: 1
  },
  // dl: 'DescriptionList',
  {
    match: (node) => node.tagName.toLowerCase() === "dl",
    roles: ["DescriptionList"],
    specificity: 1
  },
  // dt: 'DescriptionListTerm',
  {
    match: (node) => node.tagName.toLowerCase() === "dt",
    roles: ["DescriptionListTerm"],
    specificity: 1
  },
  // embed: 'EmbeddedObject',
  {
    match: (node) => node.tagName.toLowerCase() === "embed",
    roles: ["EmbeddedObject"],
    specificity: 1
  },
  // figcaption: 'Figcaption',
  {
    match: (node) => node.tagName.toLowerCase() === "figcaption",
    roles: ["Figcaption"],
    specificity: 1
  },
  // object: 'PluginObject',
  {
    match: (node) => node.tagName.toLowerCase() === "object",
    roles: ["PluginObject"],
    specificity: 1
  },
  // label: 'LabelText',
  {
    match: (node) => node.tagName.toLowerCase() === "label",
    roles: ["LabelText"],
    specificity: 1
  },
  // br: 'LineBreak',
  {
    match: (node) => node.tagName.toLowerCase() === "br",
    roles: ["LineBreak"],
    specificity: 1
  },
  // summary: 'DisclosureTriangle',
  {
    match: (node) => node.tagName.toLowerCase() === "summary",
    roles: ["DisclosureTriangle"],
    specificity: 1
  },
  // video: 'Video',
  {
    match: (node) => node.tagName.toLowerCase() === "video",
    roles: ["Video"],
    specificity: 1
  }
];

// src/tree/role-helpers.ts
var elementRoleList = buildElementRoleList(elementRoles);
function getImplicitAriaRoles(currentNode, {
  isListSubtree,
  isNonLandmarkSubtree
}) {
  let result = [];
  for (const { match, roles } of elementRoleList) {
    if (match(currentNode)) {
      result = [...roles];
      break;
    }
  }
  if (!isListSubtree) {
    for (const { match, roles } of nonListListItemRoles) {
      if (match(currentNode)) {
        result = [...roles];
        break;
      }
    }
  }
  if (isNonLandmarkSubtree) {
    for (const { match, roles } of nonLandmarkVirtualRoles) {
      if (match(currentNode)) {
        result = [...roles];
        break;
      }
    }
  }
  if (result.length === 0) {
    for (const { match, roles } of virtualRoles) {
      if (match(currentNode)) {
        result = [...roles];
        break;
      }
    }
  }
  return result;
}
function buildElementRoleList(elementRolesMap) {
  function makeElementSelector({ name, attributes }) {
    return `${name}${(attributes != null ? attributes : []).map(({ name: attributeName, value, constraints = [] }) => {
      const shouldNotExist = constraints.indexOf("undefined") !== -1;
      if (shouldNotExist) {
        return `:not([${attributeName}])`;
      } else if (value) {
        return `[${attributeName}="${value}"]`;
      } else {
        return `[${attributeName}]`;
      }
    }).join("")}`;
  }
  function getSelectorSpecificity({
    attributes = []
  }) {
    return attributes.length;
  }
  function bySelectorSpecificity({ specificity: leftSpecificity }, { specificity: rightSpecificity }) {
    return rightSpecificity - leftSpecificity;
  }
  function match(element) {
    let { attributes = [] } = element;
    const typeTextIndex = attributes.findIndex(
      (attribute) => attribute.value && attribute.name === "type" && attribute.value === "text"
    );
    if (typeTextIndex >= 0) {
      attributes = [
        ...attributes.slice(0, typeTextIndex),
        ...attributes.slice(typeTextIndex + 1)
      ];
    }
    const selector = makeElementSelector(__spreadProps(__spreadValues({}, element), { attributes }));
    return (node) => {
      if (typeTextIndex >= 0 && isInputElement(node) && node.type !== "text") {
        return false;
      }
      return node.matches(selector);
    };
  }
  let result = [];
  for (const [element, roles] of elementRolesMap.entries()) {
    result = [
      ...result,
      {
        match: match(element),
        roles: Array.from(roles),
        specificity: getSelectorSpecificity(element)
      }
    ];
  }
  return result.sort(bySelectorSpecificity);
}

// src/tree/compute-properties.ts
function checkBooleanAttribute(element, attribute) {
  const attributeValue = element.getAttribute(attribute);
  if (attributeValue === "true") {
    return true;
  }
  if (attributeValue === "false") {
    return false;
  }
  return void 0;
}
function computeAriaSelected(element) {
  if (isOptionElement(element)) {
    return element.selected;
  }
  return checkBooleanAttribute(element, "aria-selected");
}
function computeAriaBusy(element) {
  return element.getAttribute("aria-busy") === "true";
}
function computeAriaChecked(element) {
  if ("indeterminate" in element && element.indeterminate) {
    return void 0;
  }
  if ("checked" in element) {
    return element.checked;
  }
  return checkBooleanAttribute(element, "aria-checked");
}
function computeAriaPressed(element) {
  return checkBooleanAttribute(element, "aria-pressed");
}
function computeAriaCurrent(element) {
  var _a, _b;
  return (_b = (_a = checkBooleanAttribute(element, "aria-current")) != null ? _a : element.getAttribute("aria-current")) != null ? _b : false;
}
function computeAriaExpanded(element) {
  if (isDetailsElement(element)) {
    return element.open;
  }
  return checkBooleanAttribute(element, "aria-expanded");
}
function computeHeadingLevel(element) {
  const implicitHeadingLevels = {
    H1: 1,
    H2: 2,
    H3: 3,
    H4: 4,
    H5: 5,
    H6: 6
  };
  const ariaLevelAttribute = element.getAttribute("aria-level") && Number(element.getAttribute("aria-level"));
  return ariaLevelAttribute || implicitHeadingLevels[element.tagName];
}
function computeAriaValueNow(element) {
  const valueNow = element.getAttribute("aria-valuenow");
  return valueNow === null ? void 0 : +valueNow;
}
function computeAriaValueMax(element) {
  const valueMax = element.getAttribute("aria-valuemax");
  return valueMax === null ? void 0 : +valueMax;
}
function computeAriaValueMin(element) {
  const valueMin = element.getAttribute("aria-valuemin");
  return valueMin === null ? void 0 : +valueMin;
}
function computeAriaValueText(element) {
  const valueText = element.getAttribute("aria-valuetext");
  return valueText === null ? void 0 : valueText;
}
function computeRoles(element, context) {
  let roles = [];
  if (element.hasAttribute("role")) {
    roles = element.getAttribute("role").split(" ").slice(0, 1);
  } else {
    roles = getImplicitAriaRoles(element, context);
  }
  return roles;
}

// src/tree/accessibility-tree.ts
import {
  computeAccessibleDescription,
  computeAccessibleName,
  isInaccessible,
  isDisabled
} from "dom-accessibility-api";

// src/config.ts
var config = {
  isInaccessibleOptions: void 0
};
var getConfig = () => config;
var configToolkit = (options) => {
  Object.assign(config, options);
};

// src/tree/context.ts
var isNonLandmarkRole = (element, role) => ["article", "aside", "main", "nav", "section"].includes(
  element.tagName.toLowerCase()
) || ["aricle", "complementary", "main", "navigation", "region"].includes(role);
var isList = (role) => role === "list";
var isClosedDetails = (element) => isDetailsElement(element) && !element.open;

// src/tree/overrides.ts
var isContentInsideClosedDetails = (element) => {
  const parent = element.parentElement;
  if (!parent) {
    return false;
  }
  return isClosedDetails(parent) && (element instanceof HTMLElement ? !isSummaryElement(element) : true);
};
var isInaccessibleOverride = (element) => {
  return isContentInsideClosedDetails(element);
};

// src/tree/accessibility-tree.ts
var defaultOptions = {
  isListSubtree: false,
  isClosedDetailsSubtree: false,
  isNonLandmarkSubtree: false
};
var getAccessibilityTree = (element, {
  isListSubtree: userListSubtree = defaultOptions.isListSubtree,
  isClosedDetailsSubtree: userIsClosedDetailsSubtree = defaultOptions.isClosedDetailsSubtree,
  isNonLandmarkSubtree: userNonLandmarkSubtree = defaultOptions.isNonLandmarkSubtree,
  isInaccessibleOptions = getConfig().isInaccessibleOptions
} = defaultOptions) => {
  function assembleTree(element2, context) {
    if (isInaccessible(element2, context.isInaccessibleOptions) || isInaccessibleOverride(element2)) {
      return null;
    }
    const [role] = computeRoles(element2, context);
    const childNodes = Array.from(element2.childNodes);
    return {
      element: element2,
      role,
      name: computeAccessibleName(element2),
      description: computeAccessibleDescription(element2),
      state: {
        busy: computeAriaBusy(element2),
        checked: computeAriaChecked(element2),
        current: computeAriaCurrent(element2),
        disabled: isDisabled(element2),
        expanded: computeAriaExpanded(element2),
        pressed: computeAriaPressed(element2),
        selected: computeAriaSelected(element2)
      },
      queries: {
        level: computeHeadingLevel(element2),
        value: {
          min: computeAriaValueMin(element2),
          max: computeAriaValueMax(element2),
          now: computeAriaValueNow(element2),
          text: computeAriaValueText(element2)
        }
      },
      children: (childNodes != null ? childNodes : []).map((child) => {
        if (child instanceof HTMLElement) {
          return assembleTree(child, {
            isListSubtree: context.isListSubtree || isList(role),
            isClosedDetailsSubtree: context.isClosedDetailsSubtree || isClosedDetails(element2),
            isNonLandmarkSubtree: context.isNonLandmarkSubtree || isNonLandmarkRole(element2, role),
            isInaccessibleOptions
          });
        }
        if (child instanceof Text) {
          if (child.textContent === null || isInaccessibleOverride(child)) {
            return void 0;
          }
          return new StaticText(child.textContent);
        }
        return void 0;
      }).filter(isDefined)
    };
  }
  return assembleTree(element, {
    isListSubtree: userListSubtree,
    isClosedDetailsSubtree: userIsClosedDetailsSubtree,
    isNonLandmarkSubtree: userNonLandmarkSubtree,
    isInaccessibleOptions
  });
};

// src/tree/prune-container-nodes.ts
var pruneContainerNodes = (node) => {
  const flatten = (children) => {
    return children.flatMap(
      (child) => {
        if (child instanceof StaticText) {
          return [child];
        } else if (isContainer(child) && child.children) {
          return flatten(child.children);
        } else if (child.children) {
          return [__spreadProps(__spreadValues({}, child), { children: flatten(child.children) })];
        } else {
          return [child];
        }
      }
    );
  };
  return __spreadProps(__spreadValues({}, node), { children: node.children ? flatten(node.children) : [] });
};

// src/index.ts
import { isSubtreeInaccessible } from "dom-accessibility-api";
export {
  byRole,
  configToolkit,
  getAccessibilityTree,
  getConfig,
  getPrettyTree,
  isSubtreeInaccessible,
  pruneContainerNodes
};
//# sourceMappingURL=index.mjs.map